import axios from 'axios';
import { REACT_APP_API_URL } from './helpers/config';
import { Anomaly, AnomalyDetectionStatus, GraphResponse, MetadataFields } from './helpers/dtos';
import { isEmptyString, isNonEmptyString } from '@yugabytedb/ui-components';

// define unique names to use them as query keys
export enum QUERY_KEY {
  fetchAnamolies = 'fetchAnamolies',
  fetchGraphs = 'fetchGraphs',
  fetchQueries = 'fetchQueries',
  fetchUniverseMetadataList = 'fetchUniverseMetadataList',
  deleteUniverseMetadata = 'deleteUniverseMetadata',
  updateUniverseMetadata = 'updateUniverseMetadata',
  fetchUniverseListDetails = 'fetchUniverseListDetails',
  fetchUniverseById = 'fetchUniverseById',
  fetchAnomalyDetectionStatus = 'fetchAnomalyDetectionStatus'
}

class ApiService {
  // Fetches list of anomalies
  fetchAnamolies = (universeUuid: string, startTime?: Date | null, endTime?: Date | null, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/anomalies`;
    const params: any = {
      universe_uuid: universeUuid
    }
    if (startTime) {
      params.startTime = startTime;
    }
    if (endTime) {
      params.endTime = endTime;
    }
    return axios.get<Anomaly[]>(requestURL, {
      params: params
    }).then((res) => res.data);
  };

  fetchAnamoliesById = (universeUuid: string, anomalyUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/anomalies/${anomalyUuid}`;
    const params = {
      universe_uuid: universeUuid
    }
    return axios.get<Anomaly>(requestURL, {
      params: params
    }).then((res) => res.data);
  };

  // Fetches graphs and supporting data for troubleshooting 
  fetchGraphs = (universeUuid: String, data: any, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/graphs`;
    return axios.post<GraphResponse[]>(requestUrl, data, {
      params: {
        universe_uuid: universeUuid
        // mocked: true
      }
    }).then((resp) => resp.data);
  };

  fetchQueries = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/queries`;
    const params = {
      universe_uuid: universeUuid
    }
    return axios.get<any>(requestURL, {
      params: params
    }).then((res) => res.data);
  };
  fetchCustomerMetadataList = (apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/customer_metadata`;
    return axios.get<MetadataFields[]>(requestURL).then((res) => res.data);
  }
  fetchCustomerMetadataById = (customerUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/customer_metadata/${customerUuid}`;
    return axios.get(requestUrl).then((res) => res.data);
  };
  updateCustomerMetadata = (customerUuid: string, data: any, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/customer_metadata/${customerUuid}`;
    return axios.put(requestUrl, data).then((resp) => resp.data);
  };
  deleteCustomerMetadataById = (customerUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/customer_metadata/${customerUuid}`;
    return axios.delete(requestUrl).then((resp) => resp.data);
  };
  fetchUniverseMetadataList = (apiUrl?: string, customerUuid?: string, universeUuid?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/universe_metadata`;
    const params = {
      customer_uuid: customerUuid,
      universe_uuid: universeUuid
    }
    return axios.get<MetadataFields[]>(requestURL, { params: params }).then((res) => res.data);
  };
  updateUniverseMetadata = (universeUuid: string, data: any, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/universe_metadata/${universeUuid}`;
    return axios.put(requestUrl, data).then((resp) => resp.data);
  };
  deleteUniverseMetadata = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/universe_metadata/${universeUuid}`;
    return axios.delete(requestUrl).then((resp) => resp.data);
  };
  fetchUniverseListDetails = (apiUrl?: string, customerUuid?: string, universeUuid?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const params = {
      customer_uuid: customerUuid,
      universe_uuid: universeUuid
    }
    const requestUrl = `${baseUrl}/universe_details`;
    return axios.get(requestUrl, { params: params }).then((res) => res.data);
  };
  fetchUniverseById = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/universe_details/${universeUuid}`;
    return axios.get(requestUrl).then((res) => res.data);
  };
  fetchAnomalyDetectionStatus = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/anomaly_detection_status`;
    const params = {
      universe_uuid: universeUuid
    }
    return axios.get<AnomalyDetectionStatus>(requestURL, { params: params }).then((res) => res.data);
  }
}

export const TroubleshootAPI = new ApiService();
