import { Box, makeStyles } from '@material-ui/core';
import { useForm } from 'react-hook-form';
import { YBModal, YBInputField } from '@yugabytedb/ui-components';
import { useMutation } from 'react-query';
import { TroubleshootAPI } from '../../api';
import { MetadataFields, AttachUniversePayload } from '../../helpers/dtos';

const STATUS = {
  SUCCESS: 'success',
  ERROR: 'error'
};

const useStyles = makeStyles((theme) => ({
  flexColumn: {
    display: 'flex',
    flexDirection: 'column',
    width: '600px',
    overflow: 'auto',
    marginLeft: theme.spacing(2)
  },
  input: {
    marginTop: theme.spacing(2)
  },
  inProgressIcon: {
    color: '#1A44A5'
  },
  icon: {
    height: '14px',
    width: '14px'
  },
  root: {
    padding: `${theme.spacing(3)}px ${theme.spacing(4.5)}px`
  }
}));

interface AttachUniverseProps {
  universeUuid: string;
  customerUuid: string;
  apiUrl: string;
  open: boolean;
  onUpdateMetadata: (statusMsg: string) => void;
  onClose: () => void;
}

export const AttachUniverse = ({
  universeUuid,
  customerUuid,
  apiUrl,
  open,
  onClose,
  onUpdateMetadata
}: AttachUniverseProps) => {
  const helperClasses = useStyles();

  const formMethods = useForm<MetadataFields>({
    defaultValues: {
      id: universeUuid,
      customerId: customerUuid,
      dataMountPoints: ['/mnt/d0'],
      otherMountPoints: ['/']
    },
    mode: 'onChange',
    reValidateMode: 'onChange'
  });

  const {
    control,
    handleSubmit,
    getValues,
    reset,
    formState: { isDirty }
  } = formMethods;

  const troubleshootUniverse = useMutation(
    (payload: AttachUniversePayload) =>
      TroubleshootAPI.updateUniverseMetadata(payload.id, payload, apiUrl),
    {
      onSuccess: () => {
        onUpdateMetadata(STATUS.SUCCESS);
        onClose();
      },

      onError: () => {
        reset();
        onUpdateMetadata(STATUS.ERROR);
        onClose();
      }
    }
  );

  const handleFormSubmit = handleSubmit((formValues) => {
    const { id, customerId } = getValues();
    const payload = {
      id,
      customerId,
      dataMountPoints: ['/mnt/d0'],
      otherMountPoints: ['/']
    };
    troubleshootUniverse.mutateAsync(payload);
  });

  return (
    <YBModal
      open={open}
      onClose={onClose}
      title={'Attach Universe'}
      onSubmit={handleFormSubmit}
      cancelLabel={'Cancel'}
      submitLabel={'Attach'}
      size="md"
      titleSeparator
      enableBackdropDismiss
      dialogContentProps={{
        className: helperClasses.root,
        dividers: true
      }}
    >
      <Box className={helperClasses.flexColumn}>
        <YBInputField
          name="id"
          label={'Universe UUID'}
          type="text"
          control={control}
          disabled
          className={helperClasses.input}
          placeholder={'3ef2e0d6-ad28-453r-94n8-fb197e888f11'}
        />
        <YBInputField
          label={'Customer UUID'}
          name="customerId"
          type="text"
          control={control}
          disabled
          className={helperClasses.input}
          placeholder={'12d78d93-1381-4d3d-3421-ba76f47ba7a6'}
        />
      </Box>
    </YBModal>
  );
};
